@@ -25,6 +25,7 @@ from .issues import (
     Issue as Issue,
     IssueElement as IssueElement,
 )
+from .metrics_manager import MetricsManager as MetricsManager
 
 
 # TODO: Avoid duplicating here while still getting nice autodoc?
@@ -47,6 +48,7 @@ __all__ = [
     "IssueManager",
     "Issue",
     "IssueElement",
+    "MetricsManager",
 ]
 
 check_if_update_needed()
