@@ -16,6 +16,7 @@ import sys
 from .viridis import viridis_rgb as viridis_rgb
 from .turbo import turbo_rgb as turbo_rgb
 from .issues import IssueManager as IssueManager
+from .metrics_manager import MetricsManager as MetricsManager
 from .datasharing import (
     check_urls as check_urls,
     get_mode as get_mode,
@@ -196,6 +197,17 @@ class Client:
         """
         return IssueManager(self, project_id)
 
+    def get_metrics_manager(self, project_id: str) -> MetricsManager:
+        """Get a metrics manager object.
+
+        Args:
+            project_id (str): Project ID to manage.
+
+        Returns:
+            MetricsManager: The metrics manager object.
+        """
+        return MetricsManager(self, project_id)
+
     def get_projects(self) -> List[Dict[str, Any]]:
         """Get info about existing projects
 
@@ -561,6 +573,24 @@ class Client:
         result: List[Dict[str, Any]] = r.json()
         return result
 
+    def get_dataset(self, project_id: str, dataset_id: str) -> Dict[str, Any]:
+        """Get existing dataset for a project.
+
+        Args:
+            project_id (str): The project id.
+            dataset_id (str): dataset_id
+
+        Returns:
+            dict: The dataset info.
+        """
+        url = self.api_endpoint + "/projects/{}/datasets/{}".format(
+            project_id, dataset_id
+        )
+        r = requests_retry.get(url, headers=self._get_creds_headers())
+        raise_resp_exception_error(r)
+        result: Dict[str, Any] = r.json()
+        return result
+
     def delete_dataset(self, project_id: str, dataset_id: str) -> None:
         """Mark a dataset for deletion
 
@@ -953,7 +983,7 @@ class Client:
             else:
                 print("No errors found when checking first frame!")
 
-    def create_dataset(
+    def create_or_update_dataset(
         self,
         project_id: str,
         dataset_id: str,
@@ -967,10 +997,9 @@ class Client:
         delete_cache_files_after_upload: bool = True,
         check_first_frame: bool = True,
         external_metadata: Optional[Dict[str, Any]] = None,
-        upload_version: Optional[str] = None,
         pipeline_mode: str = "BATCH",
     ) -> None:
-        """Create a dataset with the provided data urls.
+        """Create or update a dataset with the provided data urls. Dataset must be created using STREAMING pipeline_mode to update.
 
         Args:
             project_id (str): project_id
@@ -984,10 +1013,11 @@ class Client:
             preview_first_frame (bool, optional): preview the first frame of the dataset in the webapp before continuing. Requires interaction.
             delete_cache_files_after_upload (bool, optional): flag to turn off automatic deletion of cache files after upload. Useful for ipython notebook users that reload/re-attempt uploads. Defaults to True.
             external_metadata (Optional[Dict[str, Any]], optional): A JSON object that can be used to attach metadata to the dataset itself
+            pipeline_mode (str, optional): BATCH or STREAMING
         """
 
         if pipeline_mode == "STREAMING":
-            return self._create_dataset_streaming(
+            return self._create_or_update_dataset_streaming(
                 project_id,
                 dataset_id,
                 data_url=data_url,
@@ -1000,7 +1030,6 @@ class Client:
                 delete_cache_files_after_upload=delete_cache_files_after_upload,
                 check_first_frame=check_first_frame,
                 external_metadata=external_metadata,
-                upload_version=upload_version,
             )
 
         assert_valid_name(dataset_id)
@@ -1012,7 +1041,7 @@ class Client:
             raise Exception("wait_timeout must be a datetime.timedelta object")
 
         if self.dataset_exists(project_id, dataset_id):
-            raise Exception("Dataset already exists.")
+            raise Exception("Cannot update dataset when using the BATCH pipeline.")
 
         if dataset:
             dataset._flush_to_disk()
@@ -1186,7 +1215,7 @@ class Client:
                     pbar.close()
                     print("Exceeded timeout waiting for job completion.")
 
-    def _create_dataset_streaming(
+    def create_dataset(
         self,
         project_id: str,
         dataset_id: str,
@@ -1200,9 +1229,57 @@ class Client:
         delete_cache_files_after_upload: bool = True,
         check_first_frame: bool = True,
         external_metadata: Optional[Dict[str, Any]] = None,
-        upload_version: Optional[str] = None,
+        pipeline_mode: str = "BATCH",
     ) -> None:
-        """Create a streaming dataset with the provided data urls. The dataset will be processed by the streaming pipeline and will support updates.
+        """Create or update a dataset with the provided data urls. Dataset must be created using STREAMING pipeline_mode to update. (backcompat shortcut for create_or_update_dataset(...))
+
+        Args:
+            project_id (str): project_id
+            dataset_id (str): dataset_id
+            data_url (Optional[str], optional): A URL to the serialized dataset entries.
+            embeddings_url (Optional[str], optional): A URL to the serialized dataset embeddings. Defaults to None.
+            dataset (Optional[LabeledDataset], optional): The LabeledDataset to upload.
+            wait_until_finish (bool, optional): Block until the dataset processing job finishes. This generally takes at least 5 minutes, and scales with the size of the dataset. Defaults to False.
+            wait_timeout (datetime.timedelta, optional): Maximum time to wait for. Defaults to 2 hours.
+            embedding_distance_metric (str, optional): Distance metric to use for embedding layout. Can be a member of ['euclidean', 'cosine']. Defaults to 'euclidean'.
+            preview_first_frame (bool, optional): preview the first frame of the dataset in the webapp before continuing. Requires interaction.
+            delete_cache_files_after_upload (bool, optional): flag to turn off automatic deletion of cache files after upload. Useful for ipython notebook users that reload/re-attempt uploads. Defaults to True.
+            external_metadata (Optional[Dict[str, Any]], optional): A JSON object that can be used to attach metadata to the dataset itself
+            pipeline_mode (str, optional): BATCH or STREAMING
+        """
+
+        return self.create_or_update_dataset(
+            project_id,
+            dataset_id,
+            data_url=data_url,
+            embeddings_url=embeddings_url,
+            dataset=dataset,
+            wait_until_finish=wait_until_finish,
+            wait_timeout=wait_timeout,
+            embedding_distance_metric=embedding_distance_metric,
+            preview_first_frame=preview_first_frame,
+            delete_cache_files_after_upload=delete_cache_files_after_upload,
+            check_first_frame=check_first_frame,
+            external_metadata=external_metadata,
+            pipeline_mode=pipeline_mode,
+        )
+
+    def _create_or_update_dataset_streaming(
+        self,
+        project_id: str,
+        dataset_id: str,
+        data_url: Optional[str] = None,
+        embeddings_url: Optional[str] = None,
+        dataset: Optional[LabeledDataset] = None,
+        wait_until_finish: bool = False,
+        wait_timeout: datetime.timedelta = datetime.timedelta(hours=2),
+        embedding_distance_metric: str = "euclidean",
+        preview_first_frame: bool = False,
+        delete_cache_files_after_upload: bool = True,
+        check_first_frame: bool = True,
+        external_metadata: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """INTERNAL: Create or update a streaming dataset with the provided data urls. The dataset will be processed by the streaming pipeline and will support updates.
 
         Args:
             project_id (str): project_id
@@ -1220,15 +1297,14 @@ class Client:
 
         assert_valid_name(dataset_id)
 
+        upload_version = str(uuid4())
+
         if embedding_distance_metric not in ["euclidean", "cosine"]:
             raise Exception("embedding_distance_metric must be euclidean or cosine.")
 
         if not isinstance(wait_timeout, datetime.timedelta):
             raise Exception("wait_timeout must be a datetime.timedelta object")
 
-        if self.dataset_exists(project_id, dataset_id):
-            raise Exception("Dataset already exists.")
-
         if not dataset:
             raise Exception("Please provide either a data_url or dataset argument")
 
@@ -1357,69 +1433,63 @@ class Client:
         print("Dataset Processing Initiated Successfully")
 
         if wait_until_finish:
-            print(f"Dataset Processing is waiting on workers to spin up...")
-            normalize_status = "PENDING"
-            while normalize_status == "PENDING":
-                time.sleep(10)
-                normalize_status = Client.parse_normalize_process_step_status(
-                    self.current_abstract_dataset_process_step_status(
-                        project_id, dataset_id
-                    )
+            # Wait for windows to finish on the dataset
+            print("Waiting for dataset to finish processing...")
+            all_upload_version_windows_done = False
+            start_time = datetime.datetime.now()
+            dataset_upload_version_windows = []
+            while not all_upload_version_windows_done:
+                r = requests_retry.get(
+                    self.api_endpoint
+                    + "/streaming/projects/{}/datasets/{}/upload_version/{}/windows".format(
+                        project_id,
+                        dataset_id,
+                        upload_version,
+                    ),
+                    headers=self._get_creds_headers(),
                 )
-                full_process_state, _ = self.current_dataset_process_state(
-                    project_id, dataset_id
-                )
-                if full_process_state == "FAILED":
-                    print("Dataset processing has failed. Exiting...")
-                    return
-            print(f"Dataset Processing Workers have been spun up.")
-            with tqdm(
-                total=100.0,
-                file=sys.stdout,
-                unit_scale=True,
-                desc="Dataset Processing Progress",
-            ) as pbar:  # type: tqdm[Any]
-                start_time = datetime.datetime.now()
-                processing_state = "PENDING"
-                display_processing_state: Callable[
-                    [str], str
-                ] = lambda state: f"Dataset Processing Status: {state}"
-                pbar.write(display_processing_state(processing_state))
-                while (datetime.datetime.now() - start_time) < wait_timeout:
-                    (
-                        new_processing_state,
-                        new_percent_done,
-                    ) = self.current_dataset_process_state(project_id, dataset_id)
-                    pbar.update(new_percent_done - pbar.n)
-                    if new_processing_state != processing_state:
-                        processing_state = new_processing_state
-                        pbar.write(display_processing_state(processing_state))
-                    processed = self.is_dataset_processed(
-                        project_id=project_id, dataset_id=dataset_id
+                if r.ok:
+                    dataset_upload_version_windows = r.json()
+                    all_upload_version_windows_done = len(
+                        dataset_upload_version_windows
+                    ) > 0 and all(
+                        window.get("status") == "DONE"
+                        or window.get("status") == "WINDOW_ERRORED"
+                        for window in dataset_upload_version_windows
                     )
-                    if processed:
-                        pbar.update(100.0 - pbar.n)
-                        pbar.close()
-                        print("Dataset is fully processed.")
-                        break
-                    if processing_state == "FAILED":
-                        pbar.update(100.0 - pbar.n)
-                        pbar.close()
-                        print("Dataset processing has failed. Exiting...")
-                        raw_logs = self.get_dataset_ingest_error_logs(
-                            project_id, dataset_id
-                        )
-                        formatted = self._format_error_logs(raw_logs)
-                        for entry in formatted:
-                            print(entry)
-
-                        break
-                    else:
-                        time.sleep(10)
-
                 if datetime.datetime.now() - start_time >= wait_timeout:
-                    pbar.close()
-                    print("Exceeded timeout waiting for job completion.")
+                    print("Exceeded timeout waiting for dataset to finish processing.")
+                    return
+                time.sleep(10)
+            errored_window_uuids = [
+                window.get("uuid")
+                for window in dataset_upload_version_windows
+                if window.get("status") == "WINDOW_ERRORED"
+            ]
+            if len(errored_window_uuids) > 0:
+                print(
+                    "Dataset processing has {}failed.".format(
+                        "partially "
+                        if len(errored_window_uuids)
+                        < len(dataset_upload_version_windows)
+                        else ""
+                    )
+                )
+                print(
+                    "Check project uploads to see more details and which frames need reuploading."
+                )
+                print(
+                    "Failed Upload IDs: {}".format(
+                        ", ".join(str(x) for x in errored_window_uuids)
+                    )
+                )
+                print(
+                    "{}/projects/{}/details?tab=streaming_uploads".format(
+                        self.api_endpoint[:-7], project_id
+                    )
+                )
+            else:
+                print("Dataset is fully processed.")
 
     def update_dataset_metadata(
         self, project_id: str, dataset_id: str, external_metadata: Dict[str, Any]
@@ -1444,8 +1514,7 @@ class Client:
         )
         raise_resp_exception_error(r)
 
-    # TODO: Check if the inferences already exist
-    def create_inferences(
+    def create_or_update_inferences(
         self,
         project_id: str,
         dataset_id: str,
@@ -1458,10 +1527,9 @@ class Client:
         embedding_distance_metric: str = "euclidean",
         delete_cache_files_after_upload: bool = True,
         external_metadata: Optional[Dict[str, Any]] = None,
-        upload_version: Optional[str] = None,
         pipeline_mode: str = "BATCH",
     ) -> None:
-        """Create an inference set with the provided data urls.
+        """Create or update an inference set with the provided data urls. Dataset + Inferences must be created using STREAMING pipeline_mode to update.
 
         Args:
             project_id (str): project_id
@@ -1470,17 +1538,20 @@ class Client:
             data_url (Optional[str], optional): A URL to the serialized inference entries.
             embeddings_url (Optional[str], optional): A URL to the serialized inference embeddings. Defaults to None.
             inferences (Optional[Inferences], optional): The inferences to upload.
-            wait_until_finish (bool, optional): Block until the dataset processing job finishes. This generally takes at least 5 minutes, and scales with the size of the dataset. Defaults to False.
+            wait_until_finish (bool, optional): Block until the dataset processing job finishes. This generally takes at least 5 minutes, and scales with the size of the dataset. Does not work in streaming pipeline mode. Defaults to False.
             wait_timeout (datetime.timedelta, optional): Maximum time to wait for. Defaults to 2 hours.
             embedding_distance_metric (str, optional): Distance metric to use for embedding layout. Can be a member of ['euclidean', 'cosine']. Defaults to 'euclidean'.
             delete_cache_files_after_upload (bool, optional): flag to turn off automatic deletion of cache files after upload. Useful for ipython notebook users that reload/re-attempt uploads. Defaults to True.
             external_metadata (Optional[Dict[str, Any]], optional): A JSON object that can be used to attach metadata to the inferences itself
+            pipeline_mode (str, optional): BATCH or STREAMING
         """
 
         assert_valid_name(dataset_id)
         assert_valid_name(inferences_id)
         queue_after_dataset = True
 
+        upload_version = str(uuid4())
+
         if embedding_distance_metric not in ["euclidean", "cosine"]:
             raise Exception("embedding_distance_metric must be euclidean or cosine.")
 
@@ -1498,8 +1569,10 @@ class Client:
         if self.is_dataset_processed(project_id, dataset_id) and queue_after_dataset:
             queue_after_dataset = False
 
-        if self.inferences_exists(project_id, dataset_id, inferences_id):
-            raise Exception("Inferences already exists.")
+        if pipeline_mode == "BATCH" and self.inferences_exists(
+            project_id, dataset_id, inferences_id
+        ):
+            raise Exception("Cannot update inferences when using the BATCH pipeline.")
 
         inferences_api_root = (
             self.api_endpoint
@@ -1580,6 +1653,8 @@ class Client:
         if inferences is not None:
             inferences._cleanup_temp_dir()
 
+        total_inferences_id = f"inferences_{dataset_id}_{inferences_id}"
+
         print(f"Inferences Processing Queuing...")
 
         r = requests_retry.post(
@@ -1588,8 +1663,67 @@ class Client:
         raise_resp_exception_error(r)
         print(f"Inferences Processing Queued Successfully")
 
-        if wait_until_finish:
-            total_inferences_id = f"inferences_{dataset_id}_{inferences_id}"
+        if wait_until_finish and pipeline_mode == "STREAMING":
+            # Wait for windows to finish on the inference set
+            print(f"Waiting for inferences to finish processing...")
+            all_upload_version_windows_done = False
+            start_time = datetime.datetime.now()
+            infset_upload_version_windows = []
+            while not all_upload_version_windows_done:
+                r = requests_retry.get(
+                    self.api_endpoint
+                    + "/streaming/projects/{}/datasets/{}/upload_version/{}/windows".format(
+                        project_id,
+                        total_inferences_id,
+                        upload_version,
+                    ),
+                    headers=self._get_creds_headers(),
+                )
+                if r.ok:
+                    infset_upload_version_windows = r.json()
+                    all_upload_version_windows_done = len(
+                        infset_upload_version_windows
+                    ) > 0 and all(
+                        window.get("status") == "DONE"
+                        or window.get("status") == "WINDOW_ERRORED"
+                        for window in infset_upload_version_windows
+                    )
+                if datetime.datetime.now() - start_time >= wait_timeout:
+                    print(
+                        "Exceeded timeout waiting for inferences to finish processing."
+                    )
+                    return
+                time.sleep(10)
+            errored_window_uuids = [
+                window.get("uuid")
+                for window in infset_upload_version_windows
+                if window.get("status") == "WINDOW_ERRORED"
+            ]
+            if len(errored_window_uuids) > 0:
+                print(
+                    "Inferences processing has {}failed.".format(
+                        "partially "
+                        if len(errored_window_uuids)
+                        < len(infset_upload_version_windows)
+                        else ""
+                    )
+                )
+                print(
+                    "Check project uploads to see more details and which inferences need reuploading."
+                )
+                print(
+                    "Failed Upload IDs: {}".format(
+                        ", ".join(str(x) for x in errored_window_uuids)
+                    )
+                )
+                print(
+                    "{}/projects/{}/details?tab=streaming_uploads".format(
+                        self.api_endpoint[:-7], project_id
+                    )
+                )
+            else:
+                print("Inferences are fully processed.")
+        elif wait_until_finish and pipeline_mode == "BATCH":
             print(f"Inferences Processing is waiting on workers to spin up...")
             normalize_status = "PENDING"
             while normalize_status == "PENDING":
@@ -1658,6 +1792,53 @@ class Client:
                     pbar.close()
                     print("Exceeded timeout waiting for job completion.")
 
+    def create_inferences(
+        self,
+        project_id: str,
+        dataset_id: str,
+        inferences_id: str,
+        data_url: Optional[str] = None,
+        embeddings_url: Optional[str] = None,
+        inferences: Optional[Inferences] = None,
+        wait_until_finish: bool = False,
+        wait_timeout: datetime.timedelta = datetime.timedelta(hours=2),
+        embedding_distance_metric: str = "euclidean",
+        delete_cache_files_after_upload: bool = True,
+        external_metadata: Optional[Dict[str, Any]] = None,
+        pipeline_mode: str = "BATCH",
+    ) -> None:
+        """Create or update an inference set with the provided data urls. Dataset + Inferences must be created using STREAMING pipeline_mode to update. (backcompat shortcut for create_or_update_inferences(...))
+
+        Args:
+            project_id (str): project_id
+            dataset_id (str): dataset_id
+            inferences_id (str): A unique identifier for this set of inferences.
+            data_url (Optional[str], optional): A URL to the serialized inference entries.
+            embeddings_url (Optional[str], optional): A URL to the serialized inference embeddings. Defaults to None.
+            inferences (Optional[Inferences], optional): The inferences to upload.
+            wait_until_finish (bool, optional): Block until the dataset processing job finishes. This generally takes at least 5 minutes, and scales with the size of the dataset. Does not work in streaming pipeline mode. Defaults to False.
+            wait_timeout (datetime.timedelta, optional): Maximum time to wait for. Defaults to 2 hours.
+            embedding_distance_metric (str, optional): Distance metric to use for embedding layout. Can be a member of ['euclidean', 'cosine']. Defaults to 'euclidean'.
+            delete_cache_files_after_upload (bool, optional): flag to turn off automatic deletion of cache files after upload. Useful for ipython notebook users that reload/re-attempt uploads. Defaults to True.
+            external_metadata (Optional[Dict[str, Any]], optional): A JSON object that can be used to attach metadata to the inferences itself
+            pipeline_mode (str, optional): BATCH or STREAMING
+        """
+
+        return self.create_or_update_inferences(
+            project_id,
+            dataset_id,
+            inferences_id,
+            data_url=data_url,
+            embeddings_url=embeddings_url,
+            inferences=inferences,
+            wait_until_finish=wait_until_finish,
+            wait_timeout=wait_timeout,
+            embedding_distance_metric=embedding_distance_metric,
+            delete_cache_files_after_upload=delete_cache_files_after_upload,
+            external_metadata=external_metadata,
+            pipeline_mode=pipeline_mode,
+        )
+
     # Even though this is the same implementation as `update_dataset_metadata`, we split it out
     # because users have been working with inference-specific functions
     def update_inferences_metadata(
