@@ -221,6 +221,47 @@ class LabeledFrame(BaseFrame):
             links=links,
         )
 
+    def add_label_2d_line_segment(
+        self,
+        *,
+        sensor_id: str,
+        label_id: str,
+        classification: str,
+        x1: Union[int, float],
+        y1: Union[int, float],
+        x2: Union[int, float],
+        y2: Union[int, float],
+        iscrowd: Optional[bool] = None,
+        user_attrs: Optional[Dict[str, Any]] = None,
+        links: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Add a label for a 2D line segment.
+
+        Args:
+            sensor_id (str): sensor_id
+            label_id (str): label_id which is unique across datasets and inferences.
+            classification (str): the classification string
+            x1 (int or float): The x-coord of the first vertex in pixels
+            y1 (int or float): The x-coord of the first vertex in pixels
+            x2 (int or float): The x-coord of the first vertex in pixels
+            y2 (int or float): The x-coord of the first vertex in pixels
+            iscrowd (bool, optional): Is this label marked as a crowd. Defaults to None.
+            user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
+        """
+        self._labels.add_2d_line_segment(
+            sensor_id=sensor_id,
+            label_id=label_id,
+            classification=classification,
+            x1=x1,
+            y1=y1,
+            x2=x2,
+            y2=y2,
+            confidence=None,
+            iscrowd=iscrowd,
+            user_attrs=user_attrs,
+            links=links,
+        )
+
     def add_label_2d_keypoints(
         self,
         *,
