@@ -5,6 +5,7 @@ The base frame and update frame classes.
 
 import datetime
 import json
+import numbers
 from io import IOBase
 from tempfile import NamedTemporaryFile
 import pyarrow as pa
@@ -141,6 +142,12 @@ class BaseFrame:
         if not embedding or len(embedding) > 12000:
             raise Exception("Length of embeddings should be between 0 and 12,000")
 
+        for embedding_el in embedding:
+            if not isinstance(embedding_el, numbers.Number):
+                raise Exception(
+                    f"Unexpectedly encountered a {type(embedding[0])} element. Only flat arrays of numbers are supported for embeddings."
+                )
+
         if not self.embedding:
             self.embedding = {
                 "task_id": self.frame_id,
@@ -362,37 +369,55 @@ class BaseFrame:
         self,
         *,
         sensor_id: str,
-        pointcloud_url: str,
-        intensity_url: str,
-        range_url: str,
+        pointcloud_url: str = None,
+        kitti_format_url: str = None,
+        intensity_url: str = None,
+        range_url: str = None,
         coord_frame_id: Optional[str] = None,
         date_captured: Optional[str] = None,
     ) -> None:
         """Add a point cloud sensor data point to this frame, contained in dense binary files of
-        little-endian values, similar to the raw format of KITTI lidar data.
+        little-endian values, similar to the raw format of KITTI lidar data. You can provide
+        a combination of the following values, as long as at least either kitti_format_url or
+        pointcloud_url are provided.
 
         Args:
             sensor_id (str): Sensor id
-            pointcloud_url (str): URL for the pointcloud: float32 [x1, y1, z1, x2, y2, z2, ...]
-            intensity_url (str): URL for the Intensity Pointcloud: unsigned int32 [i1, i2, ...]
-            range_url (str): URL for the Range Pointcloud: float32 [r1, r2, ...]
+            pointcloud_url (Optional[str]): URL for the pointcloud: float32 [x1, y1, z1, x2, y2, z2, ...]
+            kitti_format_url (Optional[str]): URL for the pointcloud + intensity: float32 [x1, y1, z1, i1, x2, y2, z2, i2, ...]
+            intensity_url (Optional[str]): URL for the Intensity Pointcloud: unsigned int32 [i1, i2, ...]
+            range_url (Optional[str]): URL for the Range Pointcloud: float32 [r1, r2, ...]
             coord_frame_id (Optional[str], optional): Id for the Coordinate Frame. Defaults to None.
             date_captured (Optional[str], optional): ISO formatted date. Defaults to None.
         """
         if not isinstance(sensor_id, str):
             raise Exception("sensor ids must be strings")
 
+        if pointcloud_url is None and kitti_format_url is None:
+            raise Exception(
+                "Either pointcloud_url or kitti_format_url must be provided."
+            )
+
+        if pointcloud_url is not None and kitti_format_url is not None:
+            raise Exception(
+                "Only one of pointcloud_url or kitti_format_url must be provided."
+            )
+
         if date_captured is None:
             date_captured = str(datetime.datetime.now())
 
         if coord_frame_id is None:
             coord_frame_id = "world"
 
-        data_urls = {
-            "pointcloud_url": pointcloud_url,
-            "range_url": range_url,
-            "intensity_url": intensity_url,
-        }
+        data_urls = {}
+        if pointcloud_url is not None:
+            data_urls["pointcloud_url"] = pointcloud_url
+        if kitti_format_url is not None:
+            data_urls["kitti_format_url"] = kitti_format_url
+        if range_url is not None:
+            data_urls["range_url"] = range_url
+        if intensity_url is not None:
+            data_urls["intensity_url"] = intensity_url
 
         if not self._coord_frame_exists(coord_frame_id):
             if coord_frame_id == "world":
@@ -614,6 +639,43 @@ class BaseFrame:
             }
         )
 
+    def add_video(
+        self,
+        *,
+        sensor_id: str,
+        video_url: str,
+        date_captured: Optional[str] = None,
+    ) -> None:
+        """Add a video "sensor" data to this frame.
+
+        Args:
+            sensor_id (str): The id of this sensor.
+            video_url (str): The URL to load this video data (mp4, webm, etc.).
+            date_captured (str, optional): ISO formatted date. Defaults to None.
+        """
+        if not isinstance(sensor_id, str):
+            raise Exception("sensor ids must be strings")
+
+        sensor_metadata: Dict[str, Any] = {}
+        if date_captured is None:
+            date_captured = str(datetime.datetime.now())
+
+        data_urls = {"video_url": video_url}
+
+        self._add_coordinate_frame(
+            {"coordinate_frame_id": sensor_id, "coordinate_frame_type": "VIDEO"}
+        )
+        self.sensor_data.append(
+            {
+                "coordinate_frame": sensor_id,
+                "data_urls": data_urls,
+                "date_captured": date_captured,
+                "sensor_id": sensor_id,
+                "sensor_metadata": sensor_metadata,
+                "sensor_type": "VIDEO_V0",
+            }
+        )
+
     def add_coordinate_frame_3d(
         self,
         *,
