@@ -5,6 +5,7 @@ The inference and inference frame modules.
 
 import datetime
 import json
+import numbers
 from io import IOBase
 from tempfile import NamedTemporaryFile
 import pyarrow as pa
@@ -55,6 +56,12 @@ class InferencesFrame(InferenceLabelSet):
         if not embedding or len(embedding) > 12000:
             raise Exception("Length of embeddings should be between 0 and 12,000")
 
+        for embedding_el in embedding:
+            if not isinstance(embedding_el, numbers.Number):
+                raise Exception(
+                    f"Unexpectedly encountered a {type(embedding[0])} element. Only flat arrays of numbers are supported for embeddings."
+                )
+
         if not self.embedding:
             self.embedding = {
                 "task_id": self.frame_id,
@@ -117,6 +124,48 @@ class InferencesFrame(InferenceLabelSet):
             user_attrs=user_attrs,
         )
 
+    def add_inference_2d_line_segment(
+        self,
+        *,
+        sensor_id: str,
+        label_id: str,
+        classification: str,
+        x1: Union[int, float],
+        y1: Union[int, float],
+        x2: Union[int, float],
+        y2: Union[int, float],
+        confidence: float,
+        iscrowd: Optional[bool] = None,
+        user_attrs: Optional[Dict[str, Any]] = None,
+        links: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Add an inference for a 2D line segment.
+
+        Args:
+            sensor_id (str): sensor_id
+            label_id (str): label_id which is unique across datasets and inferences.
+            classification (str): the classification string
+            x1 (int or float): The x-coord of the first vertex in pixels
+            y1 (int or float): The x-coord of the first vertex in pixels
+            x2 (int or float): The x-coord of the first vertex in pixels
+            y2 (int or float): The x-coord of the first vertex in pixels
+            iscrowd (bool, optional): Is this label marked as a crowd. Defaults to None.
+            user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
+        """
+        self.add_2d_line_segment(
+            sensor_id=sensor_id,
+            label_id=label_id,
+            classification=classification,
+            x1=x1,
+            y1=y1,
+            x2=x2,
+            y2=y2,
+            confidence=confidence,
+            iscrowd=iscrowd,
+            user_attrs=user_attrs,
+            links=links,
+        )
+
     def add_inference_text_token(
         self,
         *,
