@@ -444,12 +444,13 @@ class IssueManager:
 
         return resp_data["id"]
 
-    def get_issue(self, issue_id: str) -> Issue:
+    def get_issue(self, issue_id: str, exclude_frame_data: bool = False) -> Issue:
         """Get a specific issue in the associated project.
         This will also include all associated frame metadata associated with each element.
 
         Args:
             issue_id (str): The issue id.
+            exclude_frame_data (bool): Set to True to exclude full frame data from the issue element (e.g. to cut down on download size).
 
         Returns:
             Issue: The issue data (including frame_data, crop_data, and label_metadata).
@@ -457,11 +458,16 @@ class IssueManager:
         url = "/projects/{}/issues/{}/download_elements".format(
             self.project_id, issue_id
         )
+
         r = requests.get(
-            self.client.api_endpoint + url, headers=self.client._get_creds_headers()
+            self.client.api_endpoint + url,
+            headers=self.client._get_creds_headers(),
+            params={"get_as_gcs_file": True, "exclude_frame_data": exclude_frame_data},
         )
 
         raise_resp_exception_error(r)
+        signed_url = r.json()["signed_url"]
+        r = requests.get(signed_url)
         return self._issue_from_api_resp(r.json())
 
     def delete_issue(self, issue_id: str) -> None:
