@@ -124,6 +124,77 @@ class BaseLabelSet:
         }
 
     # Handle confidence
+    def add_2d_line_segment(
+        self,
+        *,
+        sensor_id: str,
+        label_id: str,
+        classification: str,
+        x1: Union[int, float],
+        y1: Union[int, float],
+        x2: Union[int, float],
+        y2: Union[int, float],
+        confidence: Optional[float] = None,
+        iscrowd: Optional[bool] = None,
+        user_attrs: Optional[Dict[str, Any]] = None,
+        links: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Add a 2D line segment.
+
+        Args:
+            sensor_id (str): sensor_id
+            label_id (str): label_id which is unique across datasets and inferences.
+            classification (str): the classification string
+            x1 (int or float): The x-coord of the first vertex in pixels
+            y1 (int or float): The x-coord of the first vertex in pixels
+            x2 (int or float): The x-coord of the first vertex in pixels
+            y2 (int or float): The x-coord of the first vertex in pixels
+            confidence (float): The confidence between 0.0 and 1.0 of the prediction
+            iscrowd (bool, optional): Is this label marked as a crowd. Defaults to None.
+            user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
+        """
+
+        if not isinstance(label_id, str):
+            raise Exception("label ids must be strings")
+        if "/" in label_id:
+            raise Exception("label ids cannot contain slashes (/)")
+
+        if not isinstance(classification, str):
+            raise Exception("classifications must be strings")
+
+        attrs = {"x1": x1, "y1": y1, "x2": x2, "y2": y2}
+
+        if confidence is not None:
+            if not isinstance(confidence, float):
+                raise Exception("confidence must be floats")
+            if confidence < 0.0 or confidence > 1.0:
+                raise Exception("confidence must be between 0.0 and 1.0")
+
+            attrs["confidence"] = confidence
+
+        if iscrowd is not None:
+            attrs["iscrowd"] = iscrowd
+
+        if links is not None:
+            for k, v in links.items():
+                if "link__" not in k:
+                    k = "link__" + k
+                attrs[k] = v
+
+        add_object_user_attrs(attrs, user_attrs)
+
+        self.label_data.append(
+            {
+                "uuid": label_id,
+                "linked_labels": [],
+                "label_type": "LINE_SEGMENT_2D",
+                "label": classification,
+                "label_coordinate_frame": sensor_id,
+                "attributes": attrs,
+            }
+        )
+        self._label_ids_set.add(label_id)
+
     def add_2d_bbox(
         self,
         *,
